<?php 
// this script will walk through the LibreOffice daily master build
// directories and produce an html file with a list with links to the latest builds

$error = '';
$masterUrl = 'https://dev-builds.libreoffice.org/daily/master/';
$masterBuilds = linxor(curling($masterUrl, $error), $masterUrl, $error);
$buildsMarkup = '';

if (!is_array($masterBuilds)) {
    $buildsMarkup .= $masterBuilds;
} else {
    foreach ($masterBuilds as $os => $dirs) {
        if ($os === "Win-") {
            $buildsMarkup .= "        <p>Windows: </p>\n";
        } else {
            $buildsMarkup .= "        <p>" . $os . ": </p>\n";
        }
        foreach ($dirs as $dir => $link) {
            if ($link == "") {
                $buildsMarkup .= "        <li>" . $dir . ": No builds found</li>\n";
            } else {
                $buildsMarkup .= "        <li><a href=\"" . $link . "\">" . $dir . "</a></li>\n";
            }
        }
    }
}

$pageMarkup = <<<MARKUP
<!DOCTYPE html>
<html class="no-js" lang="en-US">
    <head>
        <meta charset="utf-8">
        <title>LibreOffice master daily builds</title>
        <meta content="width=device-width, initial-scale=1" name="viewport">
    </head>
    <body>
$buildsMarkup
    </body>
</html>
MARKUP;

$htmlfile = fopen("libo_master_daily_builds.html", "w") or die("Unable to open file!");
fwrite($htmlfile, $pageMarkup);
fclose($htmlfile);

function curling($url, &$error) {
    // $error is passed by reference, so we can change it
    ob_start();  
    $debugout = fopen('php://output', 'w');

    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_VERBOSE, true);  
    curl_setopt($ch, CURLOPT_STDERR, $debugout); 
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 200);
    curl_setopt($ch, CURLOPT_TIMEOUT, 600);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    $data = curl_exec($ch);
    curl_close($ch);

    fclose($debugout);  
    $debugcurl = ob_get_clean();
    
    if ($data === false) {
        $error = $debugcurl;
        return false;
    } elseif ($httpcode == 404) {
        return 404;
    } else {
        return $data;
    }
}

function xpathFinder($data, $query) {
    // remove <thead> element, because it contains unescaped '&' characters,
    // which confuse DOMDocument
    $theadOpen = stripos($data, '<thead>');
    $theadClose = stripos($data, '</thead>') - $theadOpen + 8;
    $data = substr_replace($data, '', $theadOpen, $theadClose);
    $doc = new DOMDocument();
    $doc->loadHTML($data);
    $xpath = new DOMXpath($doc);
    return $xpath->query($query);
}

function linxor($data, $url, $error) {
    if ($data === false) {
        return '<p>Error: ' . $error . '</p>';
    } elseif ($data == 404) {
        return '<p>Error: file not found.</p>';
    }

    $elements = xpathFinder($data, "//*[@class='link']");
    $currents = array();

    if (!is_null($elements)) {
        // skip first element, Parent directory/
        for ($i = 1; $i <= $elements->length - 1; $i++) {
            $nodes = $elements[$i]->childNodes;
            foreach ($nodes as $node) {
                $value = $node->nodeValue;
                // ?C=S&O=D sorts descending by file size, so we get the biggest ones at the top
                $currents[rtrim($value, '/')] = $url . $value . 'current/?C=S&O=D';
            }
        }
    }

    $result = array( 'Win-' => [], 'Linux' => [], 'macOS' => [], 'Android' => [] );
    
    foreach ($currents as $tb => $link) {
        $current = curling($link, $error);
        $rpmKey = $tb . " (rpm)";
        $debKey = $tb . " (deb)";
        $firstHit = xpathFinder($current, "(//*[@class='link'])[2]");
        if ($firstHit->length == 0) {
            $firstValue = '';
            $linkRoot = '';
        } else {
            $firstValue = $firstHit->item(0)->nodeValue;
            $linkRoot = $url . $tb . '/current/';
        }
        $fullLink = $linkRoot . $firstValue;
        foreach ($result as $os => $value) {
            if ($os === 'Linux') {
                if (stripos($tb, $os) !== false) {
                    if (preg_match('/_rpm|_deb/i', $firstValue)) {
                        // directory contains two archives: rpm and deb
                        $secondHit = xpathFinder($current, "(//*[@class='link'])[3]");
                        $secondValue = $secondHit->item(0)->nodeValue;
                        (stripos($firstValue, "_rpm") !== false) ? $result[$os][$rpmKey] = $fullLink : $result[$os][$debKey] = $fullLink;
                        (stripos($secondValue, "_rpm") !== false) ? $result[$os][$rpmKey] = $linkRoot . $secondValue : $result[$os][$debKey] = $linkRoot . $secondValue;
                        continue;
                    }
                }
            }
            if (stripos($tb, $os) !== false) {
                $result[$os][$tb] = $fullLink;
            }
        }
    }
    return $result;
}

?>
